'''
mimeparse module, version 0.2

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>
                                                                                
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

WARNING! You need python2 for this module.
         If you don't have python2, uncoment mimeparse module
         and it's scanners from sagator config.
'''

from avlib import *
import os,re
import email,mimetypes,email.Errors,email.Parser

__all__=['mimeparse','mime_ext']

allowed_chars='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789._-'

class savemail(ascanner):
      '''
      Internal scanner for mime* scanners.
      
      This scanner saves stream into defined file.
      '''
      def __init__(self,dir):
          self.dir=dir
          self.filename='part-a'
      def scanstream(self,data):
          fp=open(os.path.join(self.dir,self.filename), 'wb')
          fp.write(data)
          fp.close()
          return '',[]

class mime_ext_int(ascanner):
      '''
      Internal scanner for mime_ext.
      
      This scanner checks defined extension of file.
      Regexp check with ignoring of case is used.
      '''
      def __init__(self,wrong_type):
          self.filename=''
          self.wrong_type=wrong_type
      def scanstream(self,data):
          if (self.wrong_type!='') and (type(self.filename)==type('string')):
            if re.search(self.wrong_type,self.filename,re.IGNORECASE):
               return 'MatchedExtension', [self.filename+' matched '+self.wrong_type+'\n']
          return '',[]

def decode_email(data,scanner):
    try:
      msg=email.message_from_string(data)
    except email.Errors.HeaderParseError,es:
      debug(0, "MimeParse: ERROR: HeaderParse: ",es)
      return '',[],[]
    files=[]
    counter=1
    vir,ret='',[]
    for part in msg.walk():
      if part.get_content_maintype()=='multipart':
        continue
      counter=counter+1
      sfn=part.get_filename()
      if sfn:
        # remove !allowed_chars from filename
        scanner.filename=''
        for chp in range(len(sfn)):
          if not sfn[chp] in allowed_chars:
            scanner.filename=scanner.filename+'_'
          else:
            scanner.filename=scanner.filename+sfn[chp]
      else:
        # no filename, make own
        try:
          ext = mimetypes.guess_extension(part.get_type())
        except:
          ext='.bin'
        if not ext:
          ext='.bin'
        scanner.filename = 'part-%03d%s' % (counter, ext)
      stream=part.get_payload(decode=1)
      if stream:
        if scanner.is_multistream:
          scanner.addstream(stream)
        else:
          v,r=scanner.scanstream(stream)
          if not vir:
            vir=v
          ret=ret+r
        files.append(scanner.filename)
    if scanner.is_multistream:
      vir,ret=scanner.scanmultistream([])
    del scanner
    return vir,ret,files

class mimeparse(interscanner):
      '''
      mimeparse interscanner
      
      Usage: mimeparse(another_scanner(...))
      
      This scanner parses MIME attached mail and send multiple of
      streams into another_scanner(). If an multiscanner is used
      as another_scanner, optimal method is used (scans all files
      at once).
      '''
      name='MimeParse()'
      def __init__(self,scanner):
          self.name="MimeParse("+scanner.name+")"
          self.scanner=scanner
      def scanstream(self,stream):
          vir,ret,filelist=decode_email(stream,self.scanner)
          return vir,ret

class mime_ext(ascanner):
      '''
      mime_ext() is a extension scanner.

      Usage: mime_ext('wrong types regular expressios')
      
      This scanner checks defined extension of file in mime attachments.
      Regexp checks with ignoring of case are used.
      You can use predefined extensions from mime_ext.EXECUTABLES,
      for example:
        mime_ext(mime_ext.EXECUTABLES)
      '''
      name='MimeExt()'
      EXECUTABLES='(\.exe|\.com|\.pif|\.lnk|\.scr|\.vbs)$'
      def __init__(self,wrong_type=''):
          self.wrong_type=wrong_type
      def scanstream(self,stream):
          vir,ret,filelist=decode_email(stream,mime_ext_int(self.wrong_type))
          return vir,ret
